//***************************************************************
// Filename: 	CLBRead.c
//
// Brief:	This program reads a CLB structure 
//		from file or flash on the suzaku board
//
// Author:	Fredrik Gravdal, 2007, NTNU
//***************************************************************
#include <stdio.h>
#include <stdlib.h>
#include "CLBRead.h"

//for printing data to screen at x86 arcitectures
unsigned int endian(unsigned int c){
	return((c&0xff)<<24)+((c&0xff00)<<8)+((c&0xff0000)>>8)+((c>>24)&0xff);
}

//reads one defined CLB from a inputfile
clb read_clb_file(FILE * clbFile, unsigned int row, unsigned int col){
	int i,j,k=0;
	unsigned int iData;
	unsigned int clbStart =  CLB_START/4;
	clb oneCLB;

	//haxx to read 16bit exstra. (0x263A/4=2446,5) this does the tric :)
	fread(&iData, 2,1, clbFile);

	//loops the file pointer to start at CLB (0,0)
	for(j=0; j < clbStart; j++){
		fread(&iData, 4,1, clbFile);
		iData=endian(iData);
	}

	for(j=0; j < (101*19*col)+(row*2); j++){
		fread(&iData, 4,1, clbFile);
		iData = endian(iData);
	}
		
	for (k=0;k<19*2;k+=2){	
				
		fread(&iData, 4,1, clbFile);
		//iData=endian(iData);
		oneCLB.iCLB[k] = iData;	
		//printf("k: %d data: %x \n", k,endian(iData));
		
		fread(&iData, 4,1, clbFile);
		//iData=endian(iData);
		oneCLB.iCLB[k+1] = iData;
		//printf("k: %d data: %x \n", k+1,endian(iData));
		
		//loop pointer to next CLB line (101*4byte)
		for(i=0;i<99;i++){
			fread(&iData, 4,1, clbFile);
		//	iData=endian(iData);
		}
	}
	return oneCLB;
}

//reads a CLB-structure from file
int read_file(int startCol, int startRow, int endCol, int endRow, char* outPutFile, char* inPutFile){

	FILE * outFile;
	FILE * inFile;
  	
	outFile = fopen (outPutFile,"wb");
	inFile = fopen (inPutFile,"rb");
 	if (outFile == NULL){printf("Error opening outFile\n"); return 0;}
 	if (inFile == NULL){printf("Error opening inFile\n"); return 0;}

	//write matrix structure and size to output file
	int data = endCol - startCol; 
	fwrite(&data, 4, 1, outFile);
	data = endRow - startRow;
	fwrite(&data, 4, 1, outFile);
	
	clb oneCLB;
	clb CLBTable[endRow-startRow+1][endCol-startCol+1];

	int col,row=0;
	printf("Reading from [%d] [%d] to [%d] [%d]\n",startCol,startRow, endCol, endRow);

	//loop col and row. save it in CLBTable[row][col]
	for(col = startCol; col <= endCol; col++){
		for(row = startRow; row <= endRow; row++){
		printf("[ %d ] [ %d ]\n",col,row);
			CLBTable[row - startRow][col - startCol] = read_clb_file(inFile, row, col);
			rewind (inFile);	
		
		}
	}	
		
	//save CLBTable to outFile
	fwrite(CLBTable, sizeof(oneCLB.iCLB[0]), sizeof(CLBTable)/sizeof(oneCLB.iCLB[0]), outFile);
	
	fclose (outFile);
	fclose (inFile);
	
	return 0;
}

//reads one CLB from flash
clb read_clb_flash(unsigned int row, unsigned int col){

	int k=0;
	unsigned int iData;
	unsigned int CLBaddr;
	clb oneCLB;
		
	CLBaddr = FLASH_START + FPGA_START + CLB_START + (row*4*2) + (col*101*4*19);

	for (k=0;k<19*2;k+=2){

		iData = (*(volatile unsigned short *) (CLBaddr   +k*2)) << 16;		
		iData|= (*(volatile unsigned short *) (CLBaddr+2 +k*2));		
		oneCLB.iCLB[k] = iData;	
		iData = (*(volatile unsigned short *) (CLBaddr+4 +k*2)) << 16;
		iData|= (*(volatile unsigned short *) (CLBaddr+6 +k*2));
		oneCLB.iCLB[k+1] = iData;
		
		CLBaddr += 100*4;
	}
	return oneCLB;
}

//reads a CLB-structure from flash
int read_flash(int startCol, int startRow, int endCol, int endRow, char* outPutFile){

	FILE * outFile;
  	
	outFile = fopen (outPutFile,"wb");
 	if (outFile == NULL){printf("Error opening outFile\n"); return 0;}

	//write matrix size to output file
	int data = endCol - startCol; 
	fwrite(&data, 4, 1, outFile);
	data = endRow - startRow;
	fwrite(&data, 4, 1, outFile);
	
	clb oneCLB;
	clb CLBTable[endRow-startRow+1][endCol-startCol+1];

	int col,row=0;
	printf("Reading from [%d] [%d] to [%d] [%d]\n",startCol,startRow, endCol, endRow);

	//loop col and row. save it in CLBTable[row][col]
	for(col = startCol; col <= endCol; col++){
		for(row = startRow; row <= endRow; row++){
			printf("[ %d ] [ %d ]\n",col,row);
			CLBTable[row - startRow][col - startCol] = read_clb_flash(row, col);		
		}
	}	
		
	//save CLBTable to outFile
	fwrite(CLBTable, sizeof(oneCLB.iCLB[0]), sizeof(CLBTable)/sizeof(oneCLB.iCLB[0]), outFile);
	
	fclose (outFile);

	return 0;
}

//main method
int main(int numArgs, char* argc[]){

	printf("......CLBRead......\n");
	
	int startCol = atoi(argc[1]);
	int startRow = atoi(argc[2]);
	int endCol = atoi(argc[3]); 
	int endRow = atoi(argc[4]);

	if(numArgs == 7){
		printf("CLB's will be read from input file\n",numArgs);	
		read_file(startCol, startRow, endCol, endRow, argc[5], argc[6]);
	}
	else if(numArgs == 6){
		printf("CLB's will be read from flash\n");	
		read_flash(startCol, startRow, endCol, endRow, argc[5]);
	}
	else{
		printf("Number of statements does not mach: %d\n", numArgs);		
		return 0;	
	}

	return 0;
}
