//***************************************************************
// Filename: 	FlashRead.c
//
// Brief:	This program writes a bitstreamfile to flash.
//
// Author:	Fredrik Gravdal, 2007, NTNU
//***************************************************************
#include <stdio.h>
#include <stdlib.h>
#include "FlashWrite.h"

typedef unsigned short __u16;

//changing little endian with big endian
#define ___swab16(x)                                                    \
        ({                                                              \
                __u16 __x = (x);                                        \
                ((__u16)(                                               \
                         (((__u16)(__x) & (__u16)0x00ffU) << 8) |       \
                         (((__u16)(__x) & (__u16)0xff00U) >> 8) ));     \
        })

//write data
#define flash_write(addr, b)                                    \
        (void)((*(volatile unsigned short *) (addr)) = ((b)))

//write command
#define flash_write_cmd(addr, b)                                        \
        (void)((*(volatile unsigned short *) (addr)) = (___swab16(b)))

//writes data to flash
void flash_program(unsigned long addr, unsigned short val) {
	
        flash_write_cmd(FLASH_START + 0xaaa, 0xaa);
	flash_write_cmd(FLASH_START + 0x554, 0x55);
	flash_write_cmd(FLASH_START + 0xaaa, 0xa0);
	flash_write(addr, val);
	while( (*(volatile unsigned short *) (addr)) != val){};
}

//erase flash sector
void flash_sector_erase(int sectorAddr)
{
	printf("Erasing sector: %x\n", sectorAddr);

        flash_write_cmd(FLASH_START + 0xaaa, 0xaa);
        flash_write_cmd(FLASH_START + 0x554, 0x55);
        flash_write_cmd(FLASH_START + 0xaaa, 0x80);
        flash_write_cmd(FLASH_START + 0xaaa, 0xaa);
        flash_write_cmd(FLASH_START + 0x554, 0x55);

	flash_write_cmd(FLASH_START + sectorAddr, 0x30);

        while ((*(volatile unsigned short *) (FLASH_START + sectorAddr + FLASH_SECTOR_SIZE - 1)) != 0xFFFF);

}

//writes a bitstreamfile to the flash
void writeFPGAPart (int start, char* fileToWrite){

	int i,size;
	unsigned char a,b;
	unsigned short tmp;
	unsigned int address = start;

	flash_sector_erase(FPGA_START);
	flash_sector_erase(FPGA_START + FLASH_SECTOR_SIZE);
	flash_sector_erase(FPGA_START + (FLASH_SECTOR_SIZE*2) );
	flash_sector_erase(FPGA_START + (FLASH_SECTOR_SIZE*3) );
	flash_sector_erase(FPGA_START + (FLASH_SECTOR_SIZE*4) );
	flash_sector_erase(FPGA_START + (FLASH_SECTOR_SIZE*5) );
	flash_sector_erase(FPGA_START + (FLASH_SECTOR_SIZE*6) );

	printf("Writing to flash \n");
	
	FILE * pFile;
	pFile = fopen (fileToWrite , "rb");

	if (pFile == NULL){
		printf("Error opening file");
   	}
	else {
		fseek (pFile, 0, SEEK_END);
    		size=ftell (pFile);
		rewind (pFile);	
		
		printf("Starting at address: %x\n", start);
		printf("Writing to address: %x\n", start+size);
		
		for(i=address;i <= start+size ; i+=2){
			a = fgetc (pFile);
			b = fgetc (pFile);
			tmp = a<<8;
			tmp|= b;
			flash_program(i, tmp);
		}
		fclose (pFile);
		printf("Finished writing..\n");
	}
}
	
//main method
int main(int numArgs, char* argc[]){
	
	writeFPGAPart (FLASH_START + FPGA_START, argc[1]);

	return 0;

}
